require 'test/unit'
require 'binarypacker'

class TestBinaryPacker < Test::Unit::TestCase

  def test_compile
    check_compile 'c', 'char'
    check_compile 's!', 'short'
    check_compile 'S!', 'unsigned short'
    check_compile 'S!', 'ushort'
    check_compile 'i', 'int'
    check_compile 'l', 'int32'
    check_compile 'L', 'unsigned int32'
    check_compile 'L', 'uint32'
    check_compile 'q', 'long long'
    check_compile 'Q', 'unsigned long long'
    check_compile 'v', 'uint16little'
    check_compile 'V', 'uint32little'
    check_compile 'n', 'uint16big'
    check_compile 'N', 'uint32big'

    check_compile 'c2', 'char[2]'
    check_compile 'c3', 'char[3]'
    check_compile 'c3', ' char [3] '
    check_compile 's!2', 'short[2]'

    check_compile 'cc', 'char;char'
    check_compile 'cc', 'char  ;   char'
    check_compile 'ccc', 'char;char;char'
    check_compile 'ccc', 'char  ;  char  ;  char'
    check_compile 's!s!', 'short;short'

    check_compile 'cc', "char\nchar"
    check_compile 'cc', "char\n\n\n\n\n\n\nchar"

    check_compile 'cc', "char;\nchar"
    check_compile 'cc', "char;;;;\nchar"
    check_compile 'cc', "char;\n;\n;char"
  end

  def check_compile(ok, fmt)
    assert_equal ok, BinaryPacker.compile(fmt)
  end

  def test_byte_length
    check_byte_length 0, ''
    check_byte_length 1, 'char'
    check_byte_length 2, 'short'
    check_byte_length 2, 'int16'
    check_byte_length 4, 'int'    # maybe
    check_byte_length 4, 'int32'
    check_byte_length 8, 'long long'
    check_byte_length 8, 'int64'

    check_byte_length 2, 'char[2]'
    check_byte_length 3, 'char[3]'
    check_byte_length 2, 'char;char'
    check_byte_length 3, 'char;char;char'
    check_byte_length 4, 'short[2]'
    check_byte_length 4, 'short;short'

    check_byte_length 0, 'int[*]'
    check_byte_length 1, 'char;int[*]'
    check_byte_length 2, 'char[2];int[*]'
  end

  def check_byte_length(ok, fmt)
    assert_equal ok, BinaryPacker.new(fmt).byte_length
  end

  def test_pack
    assert_equal "\000", BinaryPacker.new('char').pack([0])
    assert_equal "\001", BinaryPacker.new('char').pack([1])
    if BinaryPacker.little_endian?
      assert_equal "\001\000", BinaryPacker.new('short').pack([1])
    else
      assert_equal "\000\001", BinaryPacker.new('short').pack([1])
    end
    assert_equal "\001\000", BinaryPacker.new('uint16little').pack([1])
    assert_equal "\000\001", BinaryPacker.new('uint16big').pack([1])
    assert_raises(ArgumentError) {
      BinaryPacker.new('char').pack([])
    }
  end

  def test_unpack
    assert_equal [0], BinaryPacker.new('char').unpack("\000")
    assert_equal [1], BinaryPacker.new('char').unpack("\001")
    assert_equal [1], BinaryPacker.new('uint16little').unpack("\001\000")
    assert_equal [1], BinaryPacker.new('uint16big').unpack("\000\001")
    assert_raises(ArgumentError) {
      BinaryPacker.new('char').unpack("")
    }
    assert_nothing_raised {
      BinaryPacker.new('char[*]').unpack("")
    }
  end

end
